@@ -26,6 +26,7 @@
 #include <ge-support.h>
 #include <cairo.h>
 #include <math.h>
+#include <stdio.h>
 
 #define LINE_OPACITY 0.4
 #define HANDLE_OPACITY 0.38
@@ -41,10 +42,11 @@
 
 #define GET_ROUNDED_BUTTONS(style) (INDUSTRIAL_STYLE (style)->rounded_buttons)
 #define GET_REAL_OPACITY(style, orig_opacity) (CLAMP ((orig_opacity) * INDUSTRIAL_STYLE (style)->contrast, 0.0, 1.0))
+#define GET_HINT(style) (INDUSTRIAL_STYLE (style)->hint)
 
 #define IF_ROUNDED(style, rounded, otherwise) (GET_ROUNDED_BUTTONS (style) ? rounded : otherwise)
 
-static GtkStyleClass *parent_class;
+G_DEFINE_DYNAMIC_TYPE (IndustrialStyle, industrial_style, GTK_TYPE_STYLE)
 
 static void
 draw_rounded_gradient (cairo_t    *cr,
@@ -515,6 +517,7 @@ draw_slider (GtkStyle      *style,
 	/* This makes the slider overlay the stepper buttons at the sides. XXX */
 	if (widget && GE_IS_SCROLLBAR (widget)) {
 		GtkAdjustment *adjustment = GTK_RANGE (widget)->adjustment;
+
 		/* If all of these values are 0, don't do anything.  This fixes mozilla/galeon scrollbars. */
 		if (adjustment->value ||
 		    adjustment->lower ||
@@ -522,19 +525,27 @@ draw_slider (GtkStyle      *style,
 		    adjustment->step_increment ||
 		    adjustment->page_increment ||
 		    adjustment->page_size) {
-			if (adjustment->value <= adjustment->lower) {
+			if ((adjustment->value <= adjustment->lower) &&
+			    (GTK_RANGE (widget)->has_stepper_a || GTK_RANGE (widget)->has_stepper_b)) {
 				if (orientation == GTK_ORIENTATION_VERTICAL) {
-					y--;
+					if (!gtk_range_get_inverted (GTK_RANGE (widget)))
+						y--;
 					height++;
 				} else {
-					x--;
+					if (!gtk_range_get_inverted (GTK_RANGE (widget)))
+						x--;
 					width++;
 				}
 			}
-			if (adjustment->value >= adjustment->upper - adjustment->page_size) {
+			if ((adjustment->value >= adjustment->upper - adjustment->page_size) &&
+			    (GTK_RANGE (widget)->has_stepper_a || GTK_RANGE (widget)->has_stepper_b)) {
 				if (orientation == GTK_ORIENTATION_VERTICAL) {
+					if (gtk_range_get_inverted (GTK_RANGE (widget)))
+						y--;
 					height++;
 				} else {
+					if (gtk_range_get_inverted (GTK_RANGE (widget)))
+						x--;
 					width++;
 				}
 			}
@@ -584,10 +595,62 @@ real_draw_box (GtkStyle      *style,
 	 * important, for unkown stuff we fall back to flat_box. */
 	cairo_save (cr);
 
-	if (CHECK_DETAIL (detail, "button") ||
-	    CHECK_DETAIL (detail, "togglebutton") ||
-	    CHECK_DETAIL (detail, "optionmenu")) {
-		/* Draw the button border, except if this is a "default" button. */
+	if (CHECK_DETAIL (detail, "button") && ge_check_hint (GE_HINT_TREEVIEW_HEADER, GET_HINT (style), widget)) {
+		CairoColor bg;
+		CairoColor fg;
+		
+		ge_gdk_color_to_cairo (&style->bg[state_type], &bg);
+		ge_gdk_color_to_cairo (&style->fg[state_type], &fg);
+		
+		cairo_rectangle (cr, x, y, width, height);
+		ge_cairo_set_color (cr, &bg);
+		cairo_fill (cr);
+		
+		if (shadow_type == GTK_SHADOW_OUT) {
+			CairoColor inner, outer;
+			cairo_pattern_t *pattern;
+			
+			ge_gdk_color_to_cairo (&style->fg[state_type], &inner);
+			ge_gdk_color_to_cairo (&style->fg[state_type], &outer);
+			inner.a = 0;
+			outer.a = 0.1; /* XXX: Same as for buttons! */
+			
+			pattern = cairo_pattern_create_linear (x, height - 5, x, height - 1);
+			ge_cairo_pattern_add_color_stop_color (pattern, 0.0, &inner);
+			ge_cairo_pattern_add_color_stop_color (pattern, 1.0, &outer);
+			cairo_set_source (cr, pattern);
+			
+			cairo_rectangle (cr, x, height - 5, width, 4.0);
+			cairo_fill (cr);
+			cairo_pattern_destroy (pattern);
+
+
+			cairo_move_to (cr, x + 0.5, y + 0.5);
+			cairo_line_to (cr, x + width - 0.5, y + 0.5);
+			fg.a = BUTTON_BORDER_OPACITY * 0.05;
+			ge_cairo_set_color (cr, &fg);
+			cairo_stroke (cr);
+		}
+
+		cairo_move_to (cr, x + 0.5, height - 0.5);
+		cairo_line_to (cr, x + width - 0.5, height - 0.5);
+		fg.a = BUTTON_BORDER_OPACITY;
+		ge_cairo_set_color (cr, &fg);
+		cairo_stroke (cr);
+		
+		/* XXX: do not draw the line for the last column, if the headers are not resizeable. */
+		if (ge_widget_is_ltr (widget)) {
+			cairo_move_to (cr, x + width - 1.5, y + 5.5); /* 1.5 or 0.5? */
+			cairo_line_to (cr, x + width - 1.5, y + height - 5.5);
+		} else {
+			cairo_move_to (cr, x + 1.5, y + 5.5);
+			cairo_line_to (cr, x + 1.5, y + height - 5.5);
+		}
+		cairo_stroke (cr);
+		
+	} else if (CHECK_DETAIL (detail, "button") ||
+	           CHECK_DETAIL (detail, "togglebutton") ||
+	           CHECK_DETAIL (detail, "optionmenu")) {
 		CairoColor bg;
 		CairoColor fg;
 		CairoCorners corners = CR_CORNER_ALL;
@@ -604,7 +667,7 @@ real_draw_box (GtkStyle      *style,
 			fg.a = GET_REAL_OPACITY (style, BUTTON_BORDER_OPACITY);
 
 
-		if (ge_is_in_combo_box (widget)) {
+		if (ge_check_hint (GE_HINT_COMBOBOX_ENTRY, GET_HINT (style), widget)) {
 			if (ge_widget_is_ltr (widget))
 				corners = CR_CORNER_TOPRIGHT | CR_CORNER_BOTTOMRIGHT;
 			else
@@ -669,8 +732,6 @@ real_draw_box (GtkStyle      *style,
 			cairo_close_path (cr);
 			cairo_clip (cr);
 
-			/* XXX: This results in a few pixel that stay blank.
-			 * Need to rethink this one. */
 			draw_rounded_gradient (cr, x + 1, y + 1,
 					       width - 2, height - 2, 4, 0,
 					       IF_ROUNDED (style, 2.5, 0), &inner, &outer);
@@ -846,7 +907,7 @@ real_draw_box (GtkStyle      *style,
 			height -= 1;
 		}
 
-		parent_class->draw_flat_box (style, window, state_type,
+		GTK_STYLE_CLASS (industrial_style_parent_class)->draw_flat_box (style, window, state_type,
 					     shadow_type, area, widget, detail,
 					     x, y, width, height);
 	} else if (CHECK_DETAIL (detail, "slider")
@@ -868,18 +929,42 @@ real_draw_box (GtkStyle      *style,
 		else
 			draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
 					   &bevel, NULL, CR_CORNER_ALL);
-	} else {
-		CairoColor bevel, bg;
+	} else if (CHECK_DETAIL (detail, "entry")) {
 		CairoCorners corners = CR_CORNER_ALL;
+		CairoColor bevel, bg, base;
 
-		if ((GE_IS_SPIN_BUTTON (widget) && CHECK_DETAIL (detail, "entry")) ||
-		    ge_is_in_combo_box (widget)) {
+		ge_gdk_color_to_cairo (&style->bg[GTK_STATE_NORMAL], &bg);
+		ge_gdk_color_to_cairo (&style->base[state_type], &base);
+
+		/* Fill the background as it is initilized to base[NORMAL].
+		 * Relevant GTK+ bug: http://bugzilla.gnome.org/show_bug.cgi?id=513471
+		 * The fill only happens if no hint has been added by some application
+		 * that is faking GTK+ widgets. */
+		if (!widget || !g_object_get_data(G_OBJECT (widget), "transparent-bg-hint"))
+		{
+			cairo_rectangle (cr, x, y, width, height);
+			ge_cairo_set_color (cr, &bg);
+			cairo_fill (cr);
+		}
+		
+		if (ge_check_hint (GE_HINT_SPINBUTTON, GET_HINT (style), widget) ||
+		    ge_check_hint (GE_HINT_COMBOBOX_ENTRY, GET_HINT (style), widget)) {
 			/* effectively cut one side off. */
 			width += 2;
 			if (!ge_widget_is_ltr (widget))
 				x -= 2;
 		}
 		
+		
+		ge_gdk_color_to_cairo (&style->fg[state_type], &bevel);
+		bevel.a = GET_REAL_OPACITY (style, STANDARD_BORDER_OPACITY);
+
+		draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
+				   &bevel, &base, corners);
+	} else {
+		CairoColor bevel, bg;
+		CairoCorners corners = CR_CORNER_ALL;
+		
 		if (CHECK_DETAIL (detail, "spinbutton")) {
 			if (ge_widget_is_ltr (widget))
 				corners = CR_CORNER_TOPRIGHT | CR_CORNER_BOTTOMRIGHT;
@@ -897,7 +982,7 @@ real_draw_box (GtkStyle      *style,
 				draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
 						   &bevel, &bg, corners);
 			} else {
-				parent_class->draw_flat_box (style, window, state_type, shadow_type, area, widget, detail, x, y, width, height);
+				GTK_STYLE_CLASS (industrial_style_parent_class)->draw_flat_box (style, window, state_type, shadow_type, area, widget, detail, x, y, width, height);
 			}
 		} else if (shadow_type != GTK_SHADOW_NONE) {
 			draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
@@ -958,7 +1043,8 @@ draw_focus (GtkStyle      *style,
 		SANITIZE_SIZE
 		CHECK_ARGS
 
-		if (ge_is_in_combo_box (widget) || GE_IS_SPIN_BUTTON (widget)) {
+		if (ge_check_hint (GE_HINT_COMBOBOX_ENTRY, GET_HINT (style), widget) ||
+		    ge_check_hint (GE_HINT_SPINBUTTON, GET_HINT (style), widget)) {
 			if (ge_widget_is_ltr (widget))
 				corners = CR_CORNER_TOPLEFT | CR_CORNER_BOTTOMLEFT;
 			else
@@ -978,7 +1064,7 @@ draw_focus (GtkStyle      *style,
 		return;
 	}
 	
-	parent_class->draw_focus (style, window, state_type, area, widget, detail, x, y, width, height);
+	GTK_STYLE_CLASS (industrial_style_parent_class)->draw_focus (style, window, state_type, area, widget, detail, x, y, width, height);
 }
 
 static void
@@ -1066,7 +1152,6 @@ real_draw_box_gap (GtkStyle       *style,
 	/* clip the gap area */
 	cairo_save (cr);
 	cairo_set_fill_rule (cr, CAIRO_FILL_RULE_EVEN_ODD);
-	cairo_rectangle (cr, x, y, width, height);
 
 	switch (gap_side) {
 	case GTK_POS_TOP:
@@ -1086,6 +1171,8 @@ real_draw_box_gap (GtkStyle       *style,
 	ge_cairo_set_color (cr, &bg);
 	if (fill)
 		cairo_fill_preserve (cr);
+
+	cairo_rectangle (cr, x, y, width, height);
 	cairo_clip (cr);
 
 	/* reset fill rule */
@@ -1571,4 +1658,71 @@ render_icon (GtkStyle            *style,
 	return stated;
 }
 
-#include "industrial_style_versioned_code.h"
+/* Non-drawing stuff */
+
+void
+industrial_style_register_types (GTypeModule *module)
+{
+	industrial_style_register_type (module);
+}
+
+static void
+industrial_style_copy (GtkStyle * style, GtkStyle * src)
+{
+	IndustrialStyle *industrial_style = INDUSTRIAL_STYLE (style);
+	IndustrialStyle *industrial_src = INDUSTRIAL_STYLE (src);
+
+	industrial_style->contrast = industrial_src->contrast;
+	industrial_style->rounded_buttons = industrial_src->rounded_buttons;
+
+	GTK_STYLE_CLASS (industrial_style_parent_class)->copy (style, src);
+}
+
+static void
+industrial_style_init_from_rc (GtkStyle * style, GtkRcStyle * rc_style)
+{
+	IndustrialStyle *industrial_style = INDUSTRIAL_STYLE (style);
+	GTK_STYLE_CLASS (industrial_style_parent_class)->init_from_rc (style, rc_style);
+
+	industrial_style->contrast = INDUSTRIAL_RC_STYLE (rc_style)->contrast;
+	industrial_style->rounded_buttons = INDUSTRIAL_RC_STYLE (rc_style)->rounded_buttons;
+	industrial_style->hint = INDUSTRIAL_RC_STYLE (rc_style)->hint;
+}
+
+static void
+industrial_style_init (IndustrialStyle * style)
+{
+	style->contrast = 1.0;
+	style->rounded_buttons = TRUE;
+	style->hint = 0;
+}
+
+static void
+industrial_style_class_init (IndustrialStyleClass * klass)
+{
+	GtkStyleClass *style_class = GTK_STYLE_CLASS (klass);
+
+	style_class->copy = industrial_style_copy;
+	style_class->init_from_rc = industrial_style_init_from_rc;
+
+	style_class->draw_focus = draw_focus;
+
+	style_class->draw_handle = draw_handle;
+	style_class->draw_vline = draw_vline;
+	style_class->draw_hline = draw_hline;
+	style_class->draw_slider = draw_slider;
+
+	style_class->draw_check = draw_check;
+	style_class->draw_box = draw_box;
+	style_class->draw_shadow = draw_shadow;
+	style_class->draw_box_gap = draw_box_gap;
+	style_class->draw_shadow_gap = draw_shadow_gap;
+	style_class->draw_extension = draw_extension;
+	style_class->draw_option = draw_option;
+	style_class->render_icon = render_icon;
+}
+
+static void
+industrial_style_class_finalize (IndustrialStyleClass *klass)
+{
+}
