@@ -12,13 +12,6 @@ Tomas gren <stric@ing.umu.se>
 #include "thinice_rc_style.h"
 #include "thinice_misc.h"
 
-#ifndef max
-#define max(x,y) ((x)>=(y)?(x):(y))
-#endif
-#ifndef min
-#define min(x,y) ((x)<=(y)?(x):(y))
-#endif
-
 #define DRAW_ARGS    GtkStyle       *style, \
                      GdkWindow      *window, \
                      GtkStateType    state_type, \
@@ -33,24 +26,13 @@ Tomas gren <stric@ing.umu.se>
 
 #define DRAW_VARS    style, window, state_type, shadow_type, area, widget, detail, x, y, width, height
 
-static void thinice_style_init       (ThiniceStyle      *style);
-static void thinice_style_class_init (ThiniceStyleClass *klass);
+G_DEFINE_DYNAMIC_TYPE (ThiniceStyle, thinice_style, GTK_TYPE_STYLE)
 
-static void
-thinice_tab(GtkStyle * style,
-            GdkWindow * window,
-            GtkStateType state_type,
-            GtkShadowType shadow_type,
-            GdkRectangle * area,
-            GtkWidget * widget,
-            const gchar *detail,
-            gint x,
-            gint y,
-            gint width,
-            gint height,
-	    gint orientation);
-
-static GtkStyleClass *thinice_parent_style_class = NULL;
+void
+thinice_style_register_types (GTypeModule *module)
+{
+	thinice_style_register_type (module);
+}
 
 static void
 thinice_style_draw_hline(GtkStyle * style,
@@ -63,13 +45,18 @@ thinice_style_draw_hline(GtkStyle * style,
            gint x2,
            gint y)
 {
+	ThiniceStyle *thinice_style = THINICE_STYLE (style);
+	CairoColor *dark;
+        CairoColor *light;
 	cairo_t *cr;
 	
 	CHECK_ARGS
 
 	cr = ge_gdk_drawable_to_cairo (window, area);
 	
-	thinice_draw_separator(cr, TRUE, x1, y, x2-x1, 2);
+	dark = &thinice_style->color_cube.dark[state_type];
+	light = &thinice_style->color_cube.light[state_type];
+	thinice_draw_separator(cr, dark, light, TRUE, x1, y, x2-x1, 2);
 	cairo_destroy (cr);
 }
 
@@ -84,6 +71,9 @@ thinice_style_draw_vline(GtkStyle * style,
            gint y2,
            gint x)
 {
+	ThiniceStyle *thinice_style = THINICE_STYLE (style);
+	CairoColor *dark;
+	CairoColor *light;
 	cairo_t *cr;
 
 	CHECK_ARGS
@@ -93,7 +83,9 @@ thinice_style_draw_vline(GtkStyle * style,
 
 	cr = ge_gdk_drawable_to_cairo (window, area);
 	
-	thinice_draw_separator(cr, FALSE, x, y1, 2, y2-y1);
+	dark = &thinice_style->color_cube.dark[state_type];
+	light = &thinice_style->color_cube.light[state_type];
+	thinice_draw_separator(cr, dark, light, FALSE, x, y1, 2, y2-y1);
 
 	cairo_destroy (cr);
 }
@@ -151,13 +143,11 @@ thinice_style_draw_shadow(GtkStyle     *style,
 
     case GTK_SHADOW_ETCHED_IN:
     case GTK_SHADOW_ETCHED_OUT:
-	  ge_cairo_set_color(cr, color1);	
-   	  cairo_rectangle(cr, x + 1.5, y + 1.5, width - 3, height - 3);  
-	  cairo_stroke(cr);
+	  ge_cairo_set_color(cr, color1);
+   	  ge_cairo_stroke_rectangle(cr, x + 1.5, y + 1.5, width - 3, height - 3);  
 
 	  ge_cairo_set_color(cr, color2);	
-   	  cairo_rectangle(cr, x + 0.5, y + 0.5, width - 3, height - 3);  
-	  cairo_stroke(cr);
+   	  ge_cairo_stroke_rectangle(cr, x + 0.5, y + 0.5, width - 3, height - 3);  
       break;
 
     case GTK_SHADOW_IN:
@@ -181,15 +171,8 @@ thinice_style_draw_polygon(GtkStyle * style,
              gint npoints,
              gint fill)
 {
-#ifndef M_PI
-#define M_PI    3.14159265358979323846
-#endif /* M_PI */
-#ifndef M_PI_4
-#define M_PI_4  0.78539816339744830962
-#endif /* M_PI_4 */
-
-  static const gdouble pi_over_4 = M_PI_4;
-  static const gdouble pi_3_over_4 = M_PI_4 * 3;
+  static const gdouble pi_over_4 = G_PI_4;
+  static const gdouble pi_3_over_4 = G_PI_4 * 3;
 
   ThiniceStyle *thinice_style = THINICE_STYLE (style);
 
@@ -648,6 +631,8 @@ thinice_style_draw_box(GtkStyle * style,
 	{
 		GtkRequisition indicator_size;
 		GtkBorder indicator_spacing;
+		CairoColor *dark;
+		CairoColor *light;
 		gint vline_x;
  
 		if (state_type != GTK_STATE_INSENSITIVE)
@@ -663,7 +648,11 @@ thinice_style_draw_box(GtkStyle * style,
 
 		canvas = ge_gdk_drawable_to_cairo (window, area);
 
-		thinice_draw_separator(canvas, FALSE, vline_x, y + style->ythickness + 1, style->xthickness, height - 2*style->ythickness - 2);
+		dark = &thinice_style->color_cube.dark[state_type];
+		light = &thinice_style->color_cube.light[state_type];
+		thinice_draw_separator(canvas, dark, light, FALSE,
+                                       vline_x, y + style->ythickness + 1,
+                                       style->xthickness, height - 2*style->ythickness - 2);
 
 		cairo_destroy(canvas);
  
@@ -714,9 +703,7 @@ thinice_style_draw_check(GtkStyle * style,
 	{
 	  ge_cairo_set_color(cr, &thinice_style->color_cube.dark[state_type]);	
 
-   	  cairo_rectangle(cr, x + 0.5, y + 0.5, width - 1, height - 1);  
-
-	  cairo_stroke(cr);
+   	  ge_cairo_stroke_rectangle(cr, x + 0.5, y + 0.5, width - 1, height - 1);  
 	}
 	else
 	{
@@ -787,21 +774,21 @@ thinice_style_draw_option(GtkStyle * style,
     {
     case GTK_SHADOW_ETCHED_IN:
       ge_cairo_set_color(cr, color2);
-      cairo_arc(cr, centerX + 1, centerY + 1, radius + 1, 0, 2 * M_PI);
+      cairo_arc(cr, centerX + 1, centerY + 1, radius + 1, 0, 2 * G_PI);
       cairo_stroke(cr);
 
       ge_cairo_set_color(cr, color1); 
-      cairo_arc(cr, centerX, centerY, radius + 1, 0, 2 * M_PI);
+      cairo_arc(cr, centerX, centerY, radius + 1, 0, 2 * G_PI);
       cairo_stroke(cr);
 
       break;
     case GTK_SHADOW_ETCHED_OUT:
       ge_cairo_set_color(cr, color1); 
-      cairo_arc(cr, centerX - 1, centerY - 1, radius + 1, 0, 2 * M_PI);
+      cairo_arc(cr, centerX - 1, centerY - 1, radius + 1, 0, 2 * G_PI);
       cairo_stroke(cr);
 
       ge_cairo_set_color(cr, color2);
-      cairo_arc(cr, centerX, centerY, radius + 1,  0, 2 * M_PI);
+      cairo_arc(cr, centerX, centerY, radius + 1,  0, 2 * G_PI);
       cairo_stroke(cr);
 
       break;
@@ -811,8 +798,8 @@ thinice_style_draw_option(GtkStyle * style,
       cairo_new_path (cr);
 
       cairo_move_to(cr, centerX + (radius + 2), centerY + (radius + 2));
-      cairo_line_to(cr, centerX + (radius + 2)*sin(M_PI/4.0), centerY - (radius + 2)*cos(M_PI/4.0));
-      cairo_line_to(cr, centerX - (radius + 2)*sin(M_PI/4.0), centerY + (radius + 2)*cos(M_PI/4.0));
+      cairo_line_to(cr, centerX + (radius + 2)*sin(G_PI/4.0), centerY - (radius + 2)*cos(G_PI/4.0));
+      cairo_line_to(cr, centerX - (radius + 2)*sin(G_PI/4.0), centerY + (radius + 2)*cos(G_PI/4.0));
       cairo_line_to(cr, centerX + (radius + 2), centerY + (radius + 2));
 
       cairo_close_path (cr);
@@ -821,7 +808,7 @@ thinice_style_draw_option(GtkStyle * style,
       cairo_clip (cr);
 
       ge_cairo_set_color(cr, color2);
-      cairo_arc(cr, centerX, centerY, radius + 1, 0,  2*M_PI);
+      cairo_arc(cr, centerX, centerY, radius + 1, 0,  2*G_PI);
       cairo_fill(cr);
 
       cairo_restore(cr);
@@ -829,8 +816,8 @@ thinice_style_draw_option(GtkStyle * style,
       cairo_new_path (cr);
 
       cairo_move_to(cr, centerX - (radius + 2), centerY - (radius + 2));
-      cairo_line_to(cr, centerX + (radius + 2)*sin(M_PI/4.0), centerY - (radius + 2)*cos(M_PI/4.0));
-      cairo_line_to(cr, centerX - (radius + 2)*sin(M_PI/4.0), centerY + (radius + 2)*cos(M_PI/4.0));
+      cairo_line_to(cr, centerX + (radius + 2)*sin(G_PI/4.0), centerY - (radius + 2)*cos(G_PI/4.0));
+      cairo_line_to(cr, centerX - (radius + 2)*sin(G_PI/4.0), centerY + (radius + 2)*cos(G_PI/4.0));
       cairo_line_to(cr, centerX - (radius + 2), centerY - (radius + 2));
 
       cairo_close_path (cr);
@@ -840,13 +827,13 @@ thinice_style_draw_option(GtkStyle * style,
       cairo_clip (cr);
 
       ge_cairo_set_color(cr, color1); 
-      cairo_arc(cr, centerX, centerY, radius + 1, 0,  2*M_PI);
+      cairo_arc(cr, centerX, centerY, radius + 1, 0,  2*G_PI);
       cairo_fill(cr);
 
       cairo_restore(cr);
 
       ge_cairo_set_color(cr, color3);
-      cairo_arc(cr, centerX, centerY, radius, 0, 2 * M_PI);
+      cairo_arc(cr, centerX, centerY, radius, 0, 2 * G_PI);
       cairo_fill(cr);
 
       break;
@@ -875,6 +862,7 @@ thinice_style_draw_shadow_gap(GtkStyle * style,
 
 	CairoColor              *color1 = NULL, *color2 = NULL;
 	cairo_t *cr;
+	gint start;
 
 	CHECK_ARGS
 	SANITIZE_SIZE
@@ -900,99 +888,30 @@ thinice_style_draw_shadow_gap(GtkStyle * style,
 
 	cr = ge_gdk_drawable_to_cairo (window, area);
 
+	cairo_set_fill_rule (cr, CAIRO_FILL_RULE_EVEN_ODD);
+	cairo_rectangle (cr, x, y, width, height);
+	
 	switch (gap_side) {
         case GTK_POS_TOP:
-		if (gap_x > 0) {
-			ge_cairo_line (cr, color1, 
-				       x, y, 
-				       x + gap_x, y);
-		}
-		if ((width - (gap_x + gap_width)) > 0) {
-			ge_cairo_line (cr, color1, 
-				       x + gap_x + gap_width - 1, y,
-				       x + width - 1, y);
-		}
-		ge_cairo_line (cr, color1, 
-			       x, y, 
-			       x, y + height - 1);
-		ge_cairo_line (cr, color2,
-			       x + width - 1, y,
-			       x + width - 1, y + height - 1);
-		ge_cairo_line (cr, color2,
-			       x, y + height - 1,
-			       x + width - 1, y + height - 1);
+        	start = MAX (1, gap_x + 1);
+        	cairo_rectangle (cr, x + start, y, MIN(width-1, gap_x+gap_width - 1) - start, 1);
 		break;
         case GTK_POS_BOTTOM:
-		ge_cairo_line (cr, color1,
-			       x, y,
-			       x + width - 1, y);
-		ge_cairo_line (cr, color1, 
-			       x, y, 
-			       x, y + height - 1);
-		ge_cairo_line (cr, color2,
-			       x + width - 1, y,
-			       x + width - 1, y + height - 1);
-
-		if (gap_x > 0) {
-			ge_cairo_line (cr, color2, 
-				       x, y + height - 1, 
-				       x + gap_x, y + height - 1);
-		}
-		if ((width - (gap_x + gap_width)) > 0) {
-			ge_cairo_line (cr, color2, 
-				       x + gap_x + gap_width - 1, y + height - 1,
-				       x + width - 1, y + height - 1);
-		}
-		
+        	start = MAX (1, gap_x + 1);
+        	cairo_rectangle (cr, x + start, y + height - 1, MIN(width-1, gap_x+gap_width - 1) - start, 1);
 		break;
         case GTK_POS_LEFT:
-		ge_cairo_line (cr, color1,
-			       x, y,
-			       x + width - 1, y);
-		if (gap_x > 0) {
-			ge_cairo_line (cr, color1, 
-				       x, y,
-				       x, y + gap_x);
-		}
-		if ((height - (gap_x + gap_width)) > 0) {
-			ge_cairo_line (cr, color1, 
-				       x, y + gap_x + gap_width - 1,
-				       x, y + height - 1);
-		}
-
-		ge_cairo_line (cr, color2,
-			       x + width - 1, y,
-			       x + width - 1, y + height - 1);
-		ge_cairo_line (cr, color2,
-			       x, y + height - 1,
-			       x + width - 1, y + height - 1);
+        	start = MAX (1, gap_x + 1);
+        	cairo_rectangle (cr, x, y + start, 1, MIN(width-1, gap_x+gap_width - 1) - start);
 		break;
         case GTK_POS_RIGHT:
-		ge_cairo_line (cr, color1,
-			       x, y,
-			       x + width - 1, y);
-		ge_cairo_line (cr, color1, 
-			       x, y, 
-			       x, y + height - 1);
-
-
-		if (gap_x > 0) {
-			ge_cairo_line (cr, color2, 
-				       x + width - 1, y,
-				       x + width - 1, y + gap_x);
-		}
-		if ((height - (gap_x + gap_width)) > 0) {
-			ge_cairo_line (cr, color2, 
-				       x + width - 1, y + gap_x + gap_width - 1,
-				       x + width - 1, y + height - 1);
-		}
-
-		ge_cairo_line (cr, color2,
-			       x, y + height - 1,
-			       x + width - 1, y + height - 1);
-
+        	start = MAX (1, gap_x + 1);
+        	cairo_rectangle (cr, x + width - 1, y + start, 1, MIN(width-1, gap_x+gap_width - 1) - start);
+        	break;
 	}
-
+	cairo_clip (cr);
+	cairo_new_path (cr);
+	ge_cairo_simple_border (cr, color1, color2, x, y, width, height, FALSE);
 	cairo_destroy(cr);
 }
 
@@ -1028,15 +947,62 @@ thinice_style_draw_box_gap(GtkStyle * style,
 static void
 thinice_style_draw_extension(DRAW_ARGS, GtkPositionType gap_side)
 {
-	switch (gap_side)
+	ThiniceStyle *thinice_style = THINICE_STYLE (style);
+
+	CairoColor *background, *light, *dark;
+	cairo_t *canvas;
+
+	CHECK_ARGS
+
+	SANITIZE_SIZE
+
+	background = &thinice_style->color_cube.bg[state_type];
+	light = &thinice_style->color_cube.light[state_type];
+	dark = &thinice_style->color_cube.dark[state_type];
+
+	canvas = ge_gdk_drawable_to_cairo (window, area);
+
+	if ((!style->bg_pixmap[state_type]) || GDK_IS_PIXMAP(window))
 	{
-		case GTK_POS_TOP: gap_side = GTK_POS_BOTTOM; break;
-		case GTK_POS_BOTTOM: gap_side = GTK_POS_TOP; break;
-		case GTK_POS_LEFT: gap_side = GTK_POS_RIGHT; break;
-		case GTK_POS_RIGHT: gap_side = GTK_POS_LEFT; break;
+		ge_cairo_set_color(canvas, background);
+
+		cairo_rectangle(canvas, x, y, width, height);
+
+		cairo_fill(canvas);
+	}
+	else
+	{
+		gtk_style_apply_default_background(style, window,
+				widget && !GTK_WIDGET_NO_WINDOW(widget),
+				state_type, area, x, y, width, height);
 	}
 
-	thinice_tab(DRAW_VARS, gap_side);
+	cairo_rectangle (canvas, x, y, width, height);
+	cairo_clip (canvas);
+
+	switch(gap_side)
+	{
+		case GTK_POS_TOP:
+			y--;
+			height++;
+		break;
+
+		case GTK_POS_LEFT:
+			x--;
+			width++;
+		break;
+
+		case GTK_POS_BOTTOM:
+			height++;
+		break;
+
+		case GTK_POS_RIGHT:
+			width++;
+		break;
+	}
+
+	ge_cairo_simple_border (canvas, light, dark, x, y, width, height, FALSE);
+	cairo_destroy(canvas);
 }
 
 static void
@@ -1101,9 +1067,9 @@ thinice_style_draw_slider(GtkStyle * style,
           int chopoff;
 
           if (orientation == GTK_ORIENTATION_HORIZONTAL) {
-            chopoff = max(0, min(6, width-SMALLEST_HANDLE));
+            chopoff = CLAMP(width-SMALLEST_HANDLE, 0, 6);
           } else {
-            chopoff = max(0, min(6, height-SMALLEST_HANDLE));
+            chopoff = CLAMP(height-SMALLEST_HANDLE, 0, 6);
           }
 
           pointsh[0].x = x;                  pointsh[0].y = y+height-1;
@@ -1362,101 +1328,6 @@ thinice_style_draw_handle(GtkStyle * style,
     cairo_destroy(cr);
 }
 
-static void
-thinice_tab(DRAW_ARGS, gint orientation)
-{
-	ThiniceStyle *thinice_style = THINICE_STYLE (style);
-
-  CairoColor *light, *dark;
-  cairo_t *cr;
-
-  CHECK_ARGS
-
-  SANITIZE_SIZE
-
-  light = &thinice_style->color_cube.light[state_type];
-  dark = &thinice_style->color_cube.dark[state_type];
-
-  cr = ge_gdk_drawable_to_cairo (window, area);
-
-  if ((!style->bg_pixmap[state_type]) || GDK_IS_PIXMAP(window))
-    {
-      ge_cairo_set_color(cr, &thinice_style->color_cube.bg[state_type]);
-
-      cairo_rectangle(cr, x, y, width - 1, height - 1);
-
-      cairo_fill(cr);
-    }
-  else
-    {
-      gtk_style_apply_default_background(style, window,
-                                         widget && !GTK_WIDGET_NO_WINDOW(widget),
-                                         state_type, area, x, y, width, height);
-    }
-
-  switch(orientation)
-    {
-    case GTK_POS_TOP:
-      ge_cairo_line(cr, light,
-                    x, y + height, x, y);
-      ge_cairo_line(cr, light,
-                    x, y, x + width - 2, y);
-      ge_cairo_line(cr, dark,
-                    x + width - 2, y, x + width - 2, y + height - 1);
-      break;
-    case GTK_POS_BOTTOM:
-      ge_cairo_line(cr, light,
-                    x, y - 1, x, y + height - 1);
-      ge_cairo_line(cr, dark,
-                    x, y + height - 1, x + width - 2, y + height - 1);
-      ge_cairo_line(cr, dark,
-                    x + width - 2, y + height - 1, x + width - 2, y);
-      break;
-    case GTK_POS_LEFT:
-      ge_cairo_line(cr, light,
-                    x, y + height - 1, x, y);
-      ge_cairo_line(cr, light,
-                    x, y, x + width, y);
-      ge_cairo_line(cr, dark,
-                    x, y + height - 1, x + width - 1, y + height - 1);
-      break;
-    case GTK_POS_RIGHT:
-      ge_cairo_line(cr, light,
-                    x - 1, y, x + width - 1, y);
-      ge_cairo_line(cr, dark,
-                    x + width - 1, y, x + width - 1, y + height - 1);
-      ge_cairo_line(cr, dark,
-                    x, y + height - 1, x + width - 1, y + height - 1);
-      break;
-    }
-
-    cairo_destroy(cr);
-}
-
-GType thinice_type_style = 0;
-
-void
-thinice_style_register_type (GTypeModule *module)
-{
-  static const GTypeInfo object_info =
-  {
-    sizeof (ThiniceStyleClass),
-    (GBaseInitFunc) NULL,
-    (GBaseFinalizeFunc) NULL,
-    (GClassInitFunc) thinice_style_class_init,
-    NULL,           /* class_finalize */
-    NULL,           /* class_data */
-    sizeof (ThiniceStyle),
-    0,              /* n_preallocs */
-    (GInstanceInitFunc) thinice_style_init,
-  };
-  
-  thinice_type_style = g_type_module_register_type (module,
-						    GTK_TYPE_STYLE,
-						    "ThiniceStyle",
-						    &object_info, 0);
-}
-
 static void
 thinice_style_init (ThiniceStyle *style)
 {
@@ -1467,7 +1338,7 @@ thinice_style_realize (GtkStyle * style)
 {
   ThiniceStyle *thinice_style = THINICE_STYLE (style);
  
-  thinice_parent_style_class->realize (style);
+  GTK_STYLE_CLASS (thinice_style_parent_class)->realize (style);
  
   ge_gtk_style_to_cairo_color_cube (style, &thinice_style->color_cube);
 }
@@ -1477,8 +1348,6 @@ thinice_style_class_init (ThiniceStyleClass *klass)
 {
   GtkStyleClass *style_class = GTK_STYLE_CLASS (klass);
 
-  thinice_parent_style_class = g_type_class_peek_parent (klass);
-
   style_class->realize = thinice_style_realize;
 
   style_class->draw_hline = thinice_style_draw_hline;
@@ -1499,6 +1368,10 @@ thinice_style_class_init (ThiniceStyleClass *klass)
   style_class->draw_handle = thinice_style_draw_handle;
 }
 
+static void
+thinice_style_class_finalize (ThiniceStyleClass *klass)
+{
+}
 
 /*
 FIXME: file/font selector background OK
